//  JAWTOpenGL.java -- JAWTOpenGL
//
//  04/14/06 Oleg Kobchenko - Created.
//  Copyright 2006 Oleg Kobchenko. All rights reserved.
//

package olegyk.jawtopengl.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.Timer;
import javax.swing.*;
import java.util.*;
import olegyk.jawtopengl.*;

public class JayOpenGL extends JFrame
{
	static int offset = 0;
	Container pane = getContentPane();
	JayModel jayModel = new JayModel();
	
    public JayOpenGL() {
        super("JayOpenGL");
		// removes frame and children, exits after last window closed
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
		offset = (offset + 30) % 600;
		setLocation(offset, offset);
        setSize(500, 500);
    }
    
    public void addNotify() {
        super.addNotify();
		
		createMenu(jayModel.menu());
		
		final Label fpsLabel = new Label("0 fps");
        pane.setLayout(new BorderLayout(2, 2));
		pane.add(fpsLabel, BorderLayout.NORTH);
		pane.add(Box.createHorizontalStrut(10), BorderLayout.WEST);
		pane.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
        pane.add(jayModel, BorderLayout.CENTER);
		pane.add(new Label("  Fast, lightweight, native OpenGL"), BorderLayout.SOUTH);
		
		final Timer timer = new Timer(10, new ActionListener() {
			static final int freq = 10;
			long count = 0, time = System.currentTimeMillis();
			
			public void actionPerformed(ActionEvent evt) {
				jayModel.repaint();
				if (0 == ++count % freq) {
					long now = System.currentTimeMillis();
					fpsLabel.setText(freq*1000/(now-time) + " fps");
					time = now;
				}
			}
		});
		jayModel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (timer.isRunning()) timer.stop(); else timer.start();
			}
		});
		addWindowListener(new WindowAdapter() {
			// attempt to paint canvas before it became
			// first visible will cause "invalid drawable"
			// during [OpenGLContext setView:]
			public void windowOpened(WindowEvent e) {
				timer.start();
			}
			public void windowClosing(WindowEvent e) {
				if (timer.isRunning()) timer.stop();
			}
		});
	}
	
	void createMenu(String[][] def) {
		// JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		int ACCL = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
		JMenuBar menuBar = new JMenuBar();
		{
			JMenu menu = new JMenu("Options");
			{
				menu.add(new AbstractAction("View Definition") {
					public void actionPerformed(ActionEvent e) {
						;
					}
				});
				menu.add(new AbstractAction("Reset") {
					public void actionPerformed(ActionEvent e) {
						jayModel.reset(true);
					}
				}).setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ACCL));
			}
			menuBar.add(menu);
			
			for (int i = 0; i < def.length; i++) {
				String[] menuDef = def[i];
				menu = new JMenu(menuDef[0]);

				for (int j = 1; j < menuDef.length; j += 2) {
					if (menuDef[j] == null) {
						menu.addSeparator();
						continue;
					}
					menu.add(new AbstractAction(menuDef[j]) {
						public void actionPerformed(ActionEvent e) {
							jayModel.select(e.getActionCommand());
						}
					}).getAction().putValue(Action.ACTION_COMMAND_KEY, menuDef[j+1]);
				}
				menuBar.add(menu);
			}
		}
		setJMenuBar(menuBar);		
	}
	
    public static void main(String args[]) {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
				public void run() { new JayOpenGL().setVisible(true); }
			});
    }
	
}
